/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*----------------------------------------------------------------------*
 * Originally written:
 * 2002   Alexis <materm@tuxfamily.org>
 *
 *----------------------------------------------------------------------*/

/* $Id: notebook2.c,v 1.11 2003/04/27 12:09:15 alexis Exp $ 
 */

#include "rxvt.h"

#ifdef HAVE_LIBXPM

#include <X11/xpm.h>
#include "close_term.xpm"
#include "term.xpm"
#include "right.xpm"
#include "left.xpm"

#else

#include "close_term.xbm"
#include "term.xbm"
#include "right.xbm"
#include "left.xbm"

#endif //HAVE_LIBXPM

#ifdef DEBUG
#define DEBUG_NOT_ALEXIS 1
#else 
#define DEBUG_NOT_ALEXIS 0
#endif

#if DEBUG_NOT_ALEXIS
#define DN_ALEXIS(d,x) if(d <= DEBUG_NOT_ALEXIS) fprintf x
#else
#define DN_ALEXIS(d,x)
#endif

#define THUMB_SPACE 1    /* space between button/thumb */
#define THUMB_WIDTH 60
#define THUMB_HEIGHT 21
#define BUTTON_SIZE 22
#define BUTTON_SPACE 5

static GC whiteGC;
static GC light_greyGC; /* for the active vt*/
static GC dark_greyGC; /* unactive vt*/
static GC dark_grey2GC; /* grey for separation bettween thumbs */

enum {XPM_TERM,XPM_CLOSE,XPM_LEFT,XPM_RIGHT,NB_XPM};


#ifdef HAVE_LIBXPM
static char **xpm_name[] = {
  mini_term_xpm,close_file_xpm,
  bookmark_prev_xpm,bookmark_next_xpm
};
#else
static char *xpm_name[] = {
  term_bits,close_bits,
  left_bits,right_bits
};
#endif
  
static XImage *img[NB_XPM];
static XImage *img_mask[NB_XPM];
  
extern void create_vt();
extern char **cmd_argv;

static int width;
  
void init_notebook(void) {
  TermWin.last_page = -1;
  TermWin.active_page = -1;
  TermWin.left_page = 0;
  TermWin.right_page = -1;
  TermWin.last_active_page = 0;
}

void set_thumb_num(int page,int active) {
  int x=page-TermWin.left_page;
  char title[10];
  GC *gc;
  
  DN_ALEXIS(1,(stderr,"set_thumb_num(%d,%s)\n",page, active ? "active" : "unactive"));
  if(active) {
    gc = &light_greyGC;
  } else {
    gc = &dark_greyGC;
  }
  /* rectangle */
  XFillRectangle(Xdisplay,TermWin.thumb_bar,*gc,
    x*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,
    5,THUMB_WIDTH,THUMB_HEIGHT);

  /* top horitzontal line */
  XDrawLine(Xdisplay,TermWin.thumb_bar,whiteGC,
    x*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,5,
    x*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE+THUMB_WIDTH-1,5);
  
  if(active) {
    /* left vertical line */
    XDrawLine(Xdisplay,TermWin.thumb_bar,whiteGC,
      x*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,5,
      x*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,5+THUMB_HEIGHT-1);
    
    /* bottom horitzontal line */
    XDrawLine(Xdisplay,TermWin.thumb_bar,light_greyGC,
      x*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,5+THUMB_HEIGHT,
      (x+1)*(THUMB_WIDTH+THUMB_SPACE)/*+THUMB_SPACE+THUMB_WIDTH*/,5+THUMB_HEIGHT);
  } else {
    /* bottom horitzontal line */
    XDrawLine(Xdisplay,TermWin.thumb_bar,whiteGC,
      x*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,5+THUMB_HEIGHT,
      (x+1)*(THUMB_WIDTH+THUMB_SPACE)/*+THUMB_SPACE+THUMB_WIDTH*/,5+THUMB_HEIGHT);
  }
    
  /* right vertical line */
  XDrawLine(Xdisplay,TermWin.thumb_bar,dark_grey2GC,
    x*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE+THUMB_WIDTH,5,
    (x+1)*(THUMB_WIDTH+THUMB_SPACE)/*+THUMB_SPACE+THUMB_WIDTH*/,5+THUMB_HEIGHT-1);

  /* draw the text */
//  XSetForeground(Xdisplay,TermWin.thumbGC,PixColors[Color_Black]);

  sprintf(title,"%s",TermWin.vts[page].tab_title);
  XDrawString(Xdisplay, TermWin.thumb_bar,
    TermWin.thumbGC,x*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE+10,
    20,title,strlen(title));
    
//  XSetForeground(Xdisplay,TermWin.thumbGC,PixColors[Color_Grey25]);

}

void page_left_shift(unsigned int x) {
  DN_ALEXIS(1,(stderr,"page_right_shift(%d)\n",x));
  if( TermWin.left_page > 0 ) {
    int i;
    
    TermWin.left_page -= x; //min(0, TermWin.left_page - x);
    TermWin.right_page -= x; //max(0,TermWin.right_page - x);

    for(i=TermWin.left_page; i <= TermWin.right_page; i++) {
      if(i == TermWin.active_page) {
        set_thumb_num(i,1);
      } else {
        set_thumb_num(i,0);
      }
    }
  }
}

void page_right_shift(unsigned int x) {
  DN_ALEXIS(1,(stderr,"page_right_shift(%d)\n",x));
  if( TermWin.right_page < TermWin.last_page) {
    int i;
    
    TermWin.left_page += x; //max(TermWin.last_page,TermWin.left_page + x);
    TermWin.right_page += x; //min(TermWin.last_page,TermWin.right_page + x);

    for(i=TermWin.left_page; i <= TermWin.right_page; i++) {
      if(i == TermWin.active_page) {
        set_thumb_num(i,1);
      } else {
        set_thumb_num(i,0);
      }
    }
    
  }
}

void append_page(int index) {
  if( TermWin.last_page < MAX_PAGES-1) {
    int x; /* the top left corner of the new thumb */
    int i;
    
    DN_ALEXIS(1,(stderr,"append_page(%d)\n",index));
    
    // indicate that we add a new thumb
    TermWin.last_page++;
    
//    width = (2 * TermWin_internalBorder)+TermWin.width;
    x = TermWin.last_page*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE;
    
    DN_ALEXIS(2,(stderr,"x=%d,TermWin.width=%d\n",x,TermWin.width));
    if( x+(THUMB_WIDTH+THUMB_SPACE) > width-(4*(BUTTON_SIZE+BUTTON_SPACE)+THUMB_SPACE) ) {
      // need to shift left the thumbs
      // just compute the new value of TermWin.left_page
      DN_ALEXIS(1,(stderr,"shift on left\n"));
      TermWin.left_page = (TermWin.last_page-1) - (TermWin.right_page - TermWin.left_page) +1;
      
    }
    
    TermWin.right_page = TermWin.active_page = TermWin.last_page;
    
    /* do a left shift (if necessary) and desactivate the thumb */
    for(i=TermWin.left_page; i <= TermWin.right_page; i++) {
      set_thumb_num(i,0);
    }
    
    /* active the new active page */
    set_thumb_num(TermWin.active_page,1);
    
    create_vt();

    DN_ALEXIS(1,(stderr,"last page is %d\n",TermWin.last_page));
    f_launch_command(TermWin.last_page,cmd_argv);
    
  } else {
    DN_ALEXIS(1,(stderr,"sorry, too many tabs\n"));
  }
}

/* consider page is in [ TermWin.left_page , TermWin.right_page ]
 */
void remove_thumb(int page_num) {
  
  if( page_num >= 0 && page_num <= TermWin.last_page) {
    
//      int width = (2 * TermWin_internalBorder)+TermWin.width;
      int i;
      int minus;
      int x;
      
      if( TermWin.left_page > 0 && TermWin.right_page == TermWin.last_page ) {
        // only case where we shift on right
        
        TermWin.left_page--;
        TermWin.right_page--;
        TermWin.last_page--;

        for(i=page_num-1; i >= TermWin.left_page; i--) {
          set_thumb_num(i,0);
        }
          
      } else {
        // general case
        
        TermWin.last_page--;
        
        minus = min(TermWin.last_page,TermWin.right_page);
        
        // redraw the thumbs
        for(i=page_num; i <= minus; i++) {
          set_thumb_num(i,0);
        }
        // clear the olds thumbs
        for(; i <= TermWin.right_page; i++) {
          DN_ALEXIS(2,(stderr,"clearing thumb %d\n",i));
          x = i*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE;
          XClearArea(Xdisplay,TermWin.thumb_bar,
            x,5,THUMB_WIDTH+1,THUMB_HEIGHT,False);
        }
        
        // set right_page
        TermWin.right_page = min(TermWin.right_page,TermWin.last_page); //max(minus,TermWin.last_page);
        DN_ALEXIS(1,(stderr,"right_page=%d\n",TermWin.right_page));
      }
  }
}

// called by the handler of SIGCHLD
void remove_page(int page) {
  int the_end=0;
  
  if( TermWin.last_page == 0) {
    /* destroying the only vt => kill the app
     */
    the_end=1;
  } else if( page == TermWin.active_page) {
    if( page == 0) /* special */
      TermWin.active_page = 1;
    else 
      TermWin.active_page--;
    
    TermWin.vts[page].bMapped = 0;
    TermWin.vts[TermWin.active_page].bMapped = 1;
    XMapRaised(Xdisplay,TermWin.vts[TermWin.active_page].vt);
  }

  pat_destroy_command(page);
  pat_destroy_vt(page);

  if( TermWin.left_page <= page && page <= TermWin.right_page) {
    /* removing a visible vt */
    DN_ALEXIS(2,(stderr,"removing a visible vt\n"));
    remove_thumb(page);
  } else {
    TermWin.last_page--;
    if( page < TermWin.left_page ) {
      TermWin.left_page--;
      TermWin.right_page--;
    }
  }
  
  
  if(the_end) {
    exit(EXIT_SUCCESS);
  }
  
  if( page == 0 ) {
    TermWin.active_page = 0;
  }
  
  if( TermWin.left_page <= TermWin.active_page && TermWin.active_page <= TermWin.right_page) {
    set_thumb_num(TermWin.active_page,1);
  }
  
  DN_ALEXIS(2,(stderr,"left: %d, right: %d, last: %d, active: %d\n",
    TermWin.left_page,TermWin.right_page,TermWin.last_page,TermWin.active_page));
}

// call back appele qd on clique sur effacer un vt
// on envoie un EOF au shell, et c'est dans le sighandler
// qu'on fait le traitement.
void destroy_page(int page,pid_t pid) {
  kill(pid,9);
}


void activate_page_num(int page) {

  if(page == TermWin.active_page)
    return;
  
  TermWin.last_active_page = TermWin.active_page;
  TermWin.vts[TermWin.last_active_page].bMapped = 0;
  TermWin.active_page = page;
  TermWin.vts[page].bMapped = 1;
  
  if( TermWin.left_page <= page && page <= TermWin.right_page) {
    set_thumb_num(TermWin.active_page,1);
  }
  
  if( TermWin.left_page <= TermWin.last_active_page 
    && TermWin.last_active_page <= TermWin.right_page) {
    
    set_thumb_num(TermWin.last_active_page,0);
  }

  
  XMapRaised(Xdisplay,TermWin.vts[TermWin.active_page].vt);
//  XUnmapWindow(Xdisplay,TermWin.vts[TermWin.last_active_page].vt);
  f_scr_refresh(TermWin.active_page,FAST_REFRESH);
  DN_ALEXIS(1,(stderr,"active page is %d\n",TermWin.active_page));
  
#if defined(TRANSPARENT) || defined(BACKGROUND_IMAGE)
	refresh_transparent_scrollbar();
#endif
  
}

void resize_thumb_bar(XEvent * ev) {
  static int old_width = 0;
  int x;

  if(old_width == (ev->xconfigure).width )
    return;
  
  width = (ev->xconfigure).width;
  x = ((ev->xconfigure).width - (4*(BUTTON_SIZE+BUTTON_SPACE)+THUMB_SPACE)) / (THUMB_WIDTH+THUMB_SPACE);
  DN_ALEXIS(1,(stderr,"resize_thumb_bar, x:%d\n",x));
  
  if( (ev->xconfigure).width > old_width ) {
  
    // bigger -> may be we can add thumbs
    
    TermWin.vts[TermWin.active_page].bMapped = 0;
    TermWin.left_page = 0;
    TermWin.active_page = 0;
    TermWin.vts[TermWin.active_page].bMapped = 1;
    TermWin.right_page = min(x-1,TermWin.last_page);
    
    XMoveResizeWindow(Xdisplay,TermWin.thumb_bar,0,0,(ev->xconfigure).width,TOP_SPACE);
    
  } else if ((ev->xconfigure).width < old_width) {
  
    // smaller -> may be we need to remove thumbs
    
    TermWin.vts[TermWin.active_page].bMapped = 0;
    TermWin.left_page = 0;
    TermWin.active_page = 0;
    TermWin.vts[TermWin.active_page].bMapped = 1;
    TermWin.right_page = min(x-1,TermWin.last_page);
    
    XMoveResizeWindow(Xdisplay,TermWin.thumb_bar,0,0,(ev->xconfigure).width,TOP_SPACE);
    
  }
  
  old_width = width;
}

void dispatch_button_press(int x,int y,pid_t pid) {
  int z;
  int but;
//  int width = (2 * TermWin_internalBorder)+TermWin.width;
  
  DN_ALEXIS(2,(stderr,"click in (%d,%d)\n",x,y));
  but = -1;
  /* let's decode where the user click
   */
  z = width-x;

  if( z < 4*(BUTTON_SIZE+BUTTON_SPACE) && (z%(BUTTON_SIZE+BUTTON_SPACE)) > BUTTON_SPACE) {
    but = z/(BUTTON_SIZE+BUTTON_SPACE);
    DN_ALEXIS(1,(stderr,"click on button %d\n",but));
    switch(but) {
    case 0 : /* right shift */
      page_right_shift(1);
      break;
    case 1 : /* left shift */
      page_left_shift(1);
      break;
    case 2 : /* delete the active vt */
      destroy_page(TermWin.active_page,pid);
      break;
    case 3 : /* create e new vt*/
      append_page(0);
      break;
    default :
    break;
  }

  } else if ( x < width-(4*(BUTTON_SIZE+BUTTON_SPACE)+THUMB_SPACE) && 
    (x%(THUMB_WIDTH+THUMB_SPACE) > THUMB_SPACE ) ) {
      but = x/(THUMB_WIDTH+THUMB_SPACE) + TermWin.left_page;
      DN_ALEXIS(2,(stderr,"click on tab %d\n",but));
      if( but <= TermWin.last_page) {
        activate_page_num(but);
      }
  }
}

void thumb_bar_expose() {
  int i,j,x;
//  int width = (2 * TermWin_internalBorder) + TermWin.width;
  char title[10];
  
  DN_ALEXIS(1,(stderr,"thumb_bar_expose\n"));
  
  /* draw the buttons */
  for(i=NB_XPM; i >= 1; i--) {
    XPutImage(Xdisplay,TermWin.thumb_bar,TermWin.thumbGC,img[4-i],
      0,0,
      width-(i*(BUTTON_SIZE+BUTTON_SPACE)),5,
      BUTTON_SIZE,BUTTON_SIZE);

  }
  
  

  /* redraw the thumbs and the bevels
   */
  for(i=TermWin.left_page,x=THUMB_SPACE; i <= TermWin.right_page; i++,x += THUMB_WIDTH+THUMB_SPACE) {
    /*j = i-TermWin.left_page;*/
    
    if( i == TermWin.active_page) {
      
      XFillRectangle(Xdisplay,TermWin.thumb_bar,light_greyGC,
        x/*j*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE*/,
        5,THUMB_WIDTH,THUMB_HEIGHT);
      
      /* left vertical line */
      XDrawLine(Xdisplay,TermWin.thumb_bar,whiteGC,
        x/*j*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE*/,5,
        x/*j*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE*/,5+THUMB_HEIGHT-1);
      
    } else {
      XFillRectangle(Xdisplay,TermWin.thumb_bar,dark_greyGC,
        x/*j*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE*/,
        5,THUMB_WIDTH,THUMB_HEIGHT);
    }
    
    /* top horizontal line*/
    XDrawLine(Xdisplay,TermWin.thumb_bar,whiteGC,
      x/*j*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE*/,5,
      x/*j*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE*/+THUMB_WIDTH-1,5);
    
    /* right vertical line */
    XDrawLine(Xdisplay,TermWin.thumb_bar,dark_grey2GC,
      x/*j*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE*/+THUMB_WIDTH,5,
      x/*j*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE*/+THUMB_WIDTH,5+THUMB_HEIGHT-1);
  }
  /* draw a line under thumbs */
  XDrawLine(Xdisplay,TermWin.thumb_bar,whiteGC,0,
    5+THUMB_HEIGHT/*TOP_SPACE-1*/,
    TermWin.active_page*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,
    5+THUMB_HEIGHT/*TOP_SPACE-1*/);
  
  XDrawLine(Xdisplay,TermWin.thumb_bar,light_greyGC,
    TermWin.active_page*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,
    5+THUMB_HEIGHT/*TOP_SPACE-1*/,
    (TermWin.active_page+1)*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,
    5+THUMB_HEIGHT/*TOP_SPACE-1*/);
  
  XDrawLine(Xdisplay,TermWin.thumb_bar,whiteGC,
    (TermWin.active_page+1)*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE,
    5+THUMB_HEIGHT/*TOP_SPACE-1*/,
    width,5+THUMB_HEIGHT/*TOP_SPACE-1*/);
  
//  XSetForeground(Xdisplay,TermWin.thumbGC,PixColors[Color_Black]);
  // part for the text
  for(i=TermWin.left_page; i <= TermWin.right_page; i++) {
    j = i-TermWin.left_page;
    sprintf(title,"%s",TermWin.vts[i].tab_title);
    XDrawString(Xdisplay, TermWin.thumb_bar,
      TermWin.thumbGC,j*(THUMB_WIDTH+THUMB_SPACE)+THUMB_SPACE+10,
      20,title,strlen(title));
  }
//  XSetForeground(Xdisplay,TermWin.thumbGC,PixColors[Color_Grey25]);
  
  /* draw the bevels */
  
  XMapWindow(Xdisplay,TermWin.thumb_bar);
}

void create_thumb_bar(XSizeHints *szHint) {
  
#ifdef HAVE_LIBXPM
  XpmAttributes xpm_attr;
  XpmColorSymbol xpm_color_sym = {"background",NULL,None};
#endif
  
  XColor color;
  XGCValues gcvalue;
  Pixel darkPixel;
  int i;
  int res;

  DN_ALEXIS(1,(stderr,"Building tumb_bar\n"));
  width = szHint->width;
  /* create the light_greyGC */
  color.red = 0xd300;
  color.green = 0xd300;
  color.blue = 0xdd00;
  XAllocColor(Xdisplay,Xcmap,&color);
  gcvalue.foreground = darkPixel = color.pixel;
  
#ifdef HAVE_LIBXPM
  // this defined the transparent color of the pixmap
  xpm_color_sym.pixel = color.pixel;
#endif
  /* create the window of the thumb bar */
  TermWin.thumb_bar = XCreateSimpleWindow(
    Xdisplay,TermWin.parent,
    0,0,szHint->width,TOP_SPACE,0,
    2,color.pixel);
    
  light_greyGC = XCreateGC(Xdisplay, TermWin.thumb_bar,GCForeground , &gcvalue);
  

  /* create the dark_grey2GC */
  color.red = 0x8300;
  color.green = 0x8300;
  color.blue = 0x9300;
  XAllocColor(Xdisplay,Xcmap,&color);
  gcvalue.foreground = color.pixel;

  dark_grey2GC = XCreateGC(Xdisplay, TermWin.thumb_bar,GCForeground , &gcvalue);
  
  /* create the dark_greyGC */
  color.red = 0xc100;
  color.green = 0xc100;
  color.blue = 0xcc00;
  XAllocColor(Xdisplay,Xcmap,&color);
  gcvalue.foreground = color.pixel;
  dark_greyGC = XCreateGC(Xdisplay, TermWin.thumb_bar,GCForeground , &gcvalue);

  XSelectInput(Xdisplay,TermWin.thumb_bar,
    ExposureMask|ButtonPressMask|ButtonReleaseMask);

  /* create the GC for the thumb window 
   */
  gcvalue.foreground = PixColors[Color_Black];
	gcvalue.background = PixColors[Color_Grey25];
  gcvalue.line_width = 1;
	TermWin.thumbGC = XCreateGC(Xdisplay, TermWin.thumb_bar,
    GCForeground | GCBackground | GCLineWidth, &gcvalue);
  XSetFont(Xdisplay, TermWin.thumbGC,TermWin.font->fid);

  gcvalue.foreground = WhitePixelOfScreen(DefaultScreenOfDisplay(Xdisplay));
  whiteGC = XCreateGC(Xdisplay, TermWin.thumb_bar,
    GCForeground , &gcvalue);
  
  /* now, create the buttons
   */
#ifdef HAVE_LIBXPM
  xpm_attr.colorsymbols = &xpm_color_sym; /* to have transparent background */
  xpm_attr.numsymbols = 1;
  xpm_attr.visual = Xvisual;
  xpm_attr.colormap = Xcmap;
  xpm_attr.depth = Xdepth;
  xpm_attr.closeness = 30000;
  xpm_attr.valuemask = XpmVisual | XpmColormap | 
    XpmDepth | XpmCloseness | XpmReturnPixels | XpmColorSymbols;
#endif
    
  for(i=0; i < NB_XPM; i++) {
#ifdef HAVE_LIBXPM
    /* create the icon for the button ... */
    res = XpmCreateImageFromData(Xdisplay,xpm_name[i],
      &img[i],&img_mask[i],&xpm_attr);
    DN_ALEXIS(2,(stderr,"XpmCreateImageFromData returns %d\n",res));
#else

    img[i] = XCreateImage(Xdisplay,Xvisual,Xdepth,
      ZPixmap,0,
      xpm_name[i],
      BUTTON_SIZE,BUTTON_SIZE,8,7);
    if(img[i] == NULL) {
      printf("Null XImage\n");
    }
    
#endif
    
    /* and display it ! */
    XPutImage(Xdisplay,TermWin.thumb_bar,TermWin.thumbGC,img[i],
      0,0,
      width-((4-i)*(BUTTON_SIZE+BUTTON_SPACE)),5,
      BUTTON_SIZE,BUTTON_SIZE);
    
  }
    
  XMapWindow(Xdisplay,TermWin.thumb_bar);
}

void page_shift(unsigned int direction, unsigned int x) {
  DN_ALEXIS(2,(stderr,"page_shift(%d,%d)\n",direction,x));
  switch(direction) {
    case 0 :
      page_left_shift(x);
    break;
    case 1 :
      page_right_shift(x);
    break;
    default : DN_ALEXIS(1,(stderr,"direction not supported\n"));
  }
}
